/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wExtractor.h"
#include "wGroup.h"
#include "wSolver.h"
#include "wTag.h"
#include "wNode.h"

using namespace heat;

void Extractor::write(std::ostream &out) const
{
    out << "Extractor on " << grp;
}

Extractor::Extractor(Solver &_solver, Group &_grp) : solver(_solver), grp(_grp)
{
    // create a node list
    for (auto e : grp.tag->elems) /// cfr Dirichlet
        for (auto n : e->nodes)
            nodes.push_back(n);
    std::sort(nodes.begin(), nodes.end());
    auto it = std::unique(nodes.begin(), nodes.end());
    nodes.resize(std::distance(nodes.begin(), it));
}

std::vector<double>
Extractor::execute()
{
    std::vector<double> &vec = solver.T1; // vector of temperatures

    std::vector<double> ret;
    for (auto n : nodes)
        ret.push_back(vec[n->row]);

    return ret;
}
